var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "jrptlib/Viewer"], function (require, exports, Viewer) {
    "use strict";
    exports.__esModule = true;
    var TableBuilder = (function (_super) {
        __extends(TableBuilder, _super);
        function TableBuilder(container) {
            var _this = _super.call(this, container) || this;
            _this.sel_listeners = [];
            _this.sortColumns = [];
            _this.sortColumn = -1;
            $(container).addClass("tablebuilder");
            _this.otable = $(_this.getContainer()).find("table");
            _this.fixedHeader = $("<div>").addClass("tablebuilder-header").insertBefore(_this.otable);
            return _this;
        }
        TableBuilder.prototype.setFirstColumHeader = function (value) {
            this.firstColumnHeader = value;
        };
        TableBuilder.prototype.addSelectionListener = function (listener) {
            this.sel_listeners[this.sel_listeners.length] = listener;
        };
        TableBuilder.prototype.addSortColumn = function (column, sortMethod, upinit) {
            this.sortColumns[column] = { sortFunction: sortMethod, upward: upinit ? upinit : true };
            this.update();
            return this.sortColumns[column];
        };
        TableBuilder.prototype.setFilter = function (filter) {
            this.filter = filter;
            this.update();
        };
        TableBuilder.prototype.getFilter = function () {
            return this.filter;
        };
        TableBuilder.prototype.setAlternClass = function (classname) {
            this.alternClass = classname;
        };
        TableBuilder.prototype.checkEmptyString = function (cell, s) {
            if (s == "")
                cell.html("&nbsp;");
            else
                cell.text(s);
        };
        TableBuilder.prototype.resize = function () {
            if (this.fixedBody)
                this.fixedBody.height($(this.getContainer()).height() - $(this.fixedHeader).height());
        };
        TableBuilder.prototype.buildHeader = function (otable) {
            $(this.fixedHeader).empty();
            var tableH = $("<table>").addClass("tablebuilder-table " + $(otable).attr("class"))
                .css("margin-left", "0px")
                .appendTo(this.fixedHeader);
            $(otable).find("colgroup").clone().appendTo(tableH);
            $(otable).find("thead").clone().appendTo(tableH);
            if ($(otable).attr("id")) {
                tableH.attr("id", $(otable).attr("id"));
            }
            var headerH = $(tableH).outerHeight();
            $(otable).css("margin-top", "-" + headerH + "px");
        };
        TableBuilder.prototype.update = function () {
            var _this = this;
            if (!this.getContentProvider())
                return;
            this.buildHeader(this.otable);
            $(this.fixedBody).remove();
            this.fixedBody = $("<div>").addClass("tablebuilder-body").insertAfter(this.fixedHeader);
            $(this.otable).appendTo(this.fixedBody);
            this.resize();
            var nbColumns = $(this.otable).find("thead tr").children().length;
            var tableBuilder = this;
            if (this.sortColumns.length > 0) {
                $(this.fixedHeader).find("thead tr").children().each(function (idx) {
                    if ($(this).hasClass("sortedColumn"))
                        return;
                    if (tableBuilder.sortColumns[idx]) {
                        $(this).addClass("sortedColumn");
                        var t = $(this).text();
                        $(this).text("");
                        var link = $("<a>").attr("href", "javascript: void(0)")
                            .append($(this).children()).text(t).click(function () {
                            tableBuilder.removeColumnSortIndicator();
                            tableBuilder.sortColumn = idx;
                            tableBuilder.sortColumns[idx].upward = !tableBuilder.sortColumns[idx].upward;
                            tableBuilder.update();
                        });
                        $(link).appendTo($(this));
                    }
                });
            }
            var tablebody = $(this.otable).find("tbody").length ? $(this.otable).find("tbody") : $("<tbody>").appendTo(this.otable);
            tablebody.find("tr").remove();
            this.getContentProvider().getElements(this.getInput(), function (data) {
                if (_this.sortColumn != -1) {
                    data.sort(function (a, b) {
                        var ret = _this.sortColumns[_this.sortColumn].sortFunction(a, b);
                        ret = _this.sortColumns[_this.sortColumn].upward ? ret : -ret;
                        return ret;
                    });
                    tableBuilder.addColumnSortIndicator(_this.sortColumns[_this.sortColumn].upward);
                }
                if (_this.filter) {
                    data = data.filter(function (v) {
                        return _this.filter(v);
                    });
                }
                $(data).each(function (index) {
                    var row = $("<tr>").appendTo(tablebody);
                    if (_this.alternClass) {
                        if (index % 2 == 0)
                            row.addClass(_this.alternClass);
                    }
                    if (index == 0) {
                        $(row).on("click", function () {
                            for (var i = 0; i < _this.sel_listeners.length; i++) {
                                _this.sel_listeners[i].selectChanged(data[index]);
                            }
                        });
                    }
                    var _loop_1 = function () {
                        td = (cidx == 0 && _this.firstColumnHeader) ? $("<th>") : $("<td>");
                        td.appendTo(row);
                        if (_this.getLabelProvider().getIcon) {
                            img = $("<img>").appendTo(td);
                            img.attr("src", _this.getLabelProvider().getIcon(data[index], cidx, function (data) { img.attr("src", data); }));
                        }
                        var cell = cidx == 0 ? $("<a>") : $("<span>");
                        cell.attr("href", "javascript:void(0)").appendTo(td).click(function () {
                            for (var i = 0; i < _this.sel_listeners.length; i++) {
                                _this.sel_listeners[i].selectChanged(data[index]);
                            }
                        });
                        _this.checkEmptyString(cell, _this.getLabelProvider().getText(data[index], cidx, function (data) {
                            _this.checkEmptyString(cell, data);
                        }));
                        if (_this.getLabelProvider().getCssClass) {
                            td.addClass(_this.getLabelProvider().getCssClass(data[index], cidx, function (data) { td.addClass(data); }));
                        }
                    };
                    var td, img;
                    for (var cidx = 0; cidx < nbColumns; cidx++) {
                        _loop_1();
                    }
                });
            });
        };
        TableBuilder.prototype.removeColumnSortIndicator = function () {
            if (this.sortColumn == -1)
                return;
            $(this.getContainer()).find("thead tr .sortIndicator").remove();
        };
        TableBuilder.prototype.addColumnSortIndicator = function (upward) {
            if (this.sortColumn == -1)
                return;
            var tableBuilder = this;
            $(this.getContainer()).find("thead tr").children().each(function (idx) {
                if (idx == tableBuilder.sortColumn) {
                    $(this).find(".sortIndicator").remove();
                    $("<span>").addClass("ui-icon").attr("style", "float:right")
                        .addClass(upward ? "ui-icon-arrowthick-1-n" : "ui-icon-arrowthick-1-s")
                        .addClass("sortIndicator").appendTo($(this));
                }
            });
        };
        return TableBuilder;
    }(Viewer));
    exports.TableBuilder = TableBuilder;
});
