define(["require", "exports", "jrptlib/Nls", "jrptlib/Properties!APPMSG", "jrptlib/Url"], function (require, exports, NLS, APPMSG, Url) {
    "use strict";
    exports.__esModule = true;
    var SessionContentProvider = (function () {
        function SessionContentProvider(currentSessionSet) {
            this.currentSessionSet = currentSessionSet;
        }
        SessionContentProvider.prototype.createElements = function (container, cpath, parentPath) {
            var ret = [];
            if (container) {
                if (cpath != "") {
                    ret.push({ name: parentPath,
                        type: "Container",
                        path: parentPath,
                        isParent: true,
                        object: null,
                        label: APPMSG.SessionContentProvider_parentContainerLabel,
                        model: null });
                }
                for (var i = 0; i < container.children.length; i++) {
                    ret.push({ name: container.children[i],
                        type: "Container",
                        path: cpath,
                        object: container,
                        label: container.children[i],
                        model: container.children[i] });
                }
                for (var i = 0; i < container.sessions.length; i++) {
                    var session = container.sessions[i];
                    if (!this.alreadyLoaded(cpath + "/" + session.id)) {
                        ret.push({ name: session.id,
                            testName: session.testName,
                            label: session.label,
                            date: new Date(parseInt(session.startTime)),
                            type: "Session",
                            path: cpath,
                            object: container,
                            model: session });
                    }
                }
            }
            return ret;
        };
        SessionContentProvider.prototype.getElements = function (item, handler) {
            this.getChildren(item, handler);
        };
        SessionContentProvider.prototype.hasChildren = function (item, handler) {
            handler(item.type === "Container");
        };
        SessionContentProvider.prototype.getChildren = function (item, handler) {
            var url = "/analytics/sessions";
            var npath = "";
            if (item != null) {
                if (item.type === "Container") {
                    if (item.isParent) {
                        npath = item.path;
                        url += npath;
                    }
                    else {
                        npath = item.path + "/" + encodeURIComponent(item.name);
                        url += npath;
                    }
                }
            }
            var content_p = this;
            var nurl = new Url(url);
            nurl.get(null, null, "json").done(function (data) {
                var ppath = npath.substr(0, npath.lastIndexOf("/") != -1 ? npath.lastIndexOf("/") : npath.length);
                handler(content_p.createElements(data, npath, ppath));
            }).fail(function (jqXHR, textStatus, errorThrown) {
                _app.showErrorMessage(NLS.bind(APPMSG.Unable_Retrieve_Sessions, errorThrown));
            });
        };
        SessionContentProvider.prototype.alreadyLoaded = function (sessionPath) {
            if (!this.currentSessionSet)
                return false;
            var path = this.currentSessionSet.primarySession.sessionPath;
            if (path === sessionPath) {
                return true;
            }
            for (var i = 0; i < this.currentSessionSet.sessions.length; i++) {
                path = this.currentSessionSet.sessions[i].sessionPath;
                if (path === sessionPath) {
                    return true;
                }
            }
            return false;
        };
        return SessionContentProvider;
    }());
    exports.SessionContentProvider = SessionContentProvider;
});
