var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "ui/SessionContentProvider", "ui/SessionLabelProvider", "jrptlib/TableBuilder", "jrptlib/DateFormat", "model/session/Session", "view/query/GStatusQueryProvider", "jrptlib/Properties!APPMSG"], function (require, exports, SessionContentProvider_1, SessionLabelProvider, TableBuilder_1, DateFormat, Session, GStatusQueryProvider, APPMSG) {
    "use strict";
    var SessionSelector = (function (_super) {
        __extends(SessionSelector, _super);
        function SessionSelector(container, session_set) {
            var _this = _super.call(this, container) || this;
            _this.filter_tab = [APPMSG.COMPLETE,
                APPMSG.COMPLETED_WITH_ERROR,
                APPMSG.STOPPED_MANUALLY,
                APPMSG.INIT,
                APPMSG.PERFORMING_DATA_TRANSFERT,
                APPMSG.RUNNING];
            _this.toolbar = $("<div>").addClass("toolbar").prependTo($(_this.getContainer()));
            _this.filter_button = $("<button>").attr("title", APPMSG.Filter_button).appendTo(_this.toolbar).button().click(function () {
                _this.toggleMenu();
            });
            var session_content_provider = new SessionContentProvider_1.SessionContentProvider(session_set);
            _this.statusMap = {};
            _this.addSortColumn(SessionSelector.COL_TIME, function (a, b) {
                if (a.type === "Session" && b.type === "Session") {
                    return a.date - b.date;
                }
                else if (a.type == "Container" && b.type === "Session") {
                    return -1;
                }
                else if (a.type == "Session" && b.type === "Container") {
                    return 1;
                }
                return 0;
            });
            _this.addSortColumn(SessionSelector.COL_NAME, function (a, b) {
                if (a.type == "Container" && b.type === "Session") {
                    return -1;
                }
                else if (a.type == "Session" && b.type === "Container") {
                    return 1;
                }
                else if (a.type === "Session" && b.type === "Session") {
                    if (a.label == b.label) {
                        return b.date - a.date;
                    }
                    return a.label.localeCompare(b.label);
                }
                return 0;
            });
            _this.addSortColumn(SessionSelector.COL_STATUS, function (a, b) {
                if (a.type == "Container" && b.type === "Session") {
                    return -1;
                }
                else if (a.type === "Session" && b.type === "Container") {
                    return 1;
                }
                else if (a.type === "Session" && b.type === "Session") {
                    var status_a = _this.statusMap[a.name];
                    var status_b = _this.statusMap[b.name];
                    if (status_a)
                        return status_a.localeCompare(status_b);
                }
                return 0;
            });
            var session_selector = _this;
            _this.addSelectionListener({
                selectChanged: function (selection) {
                    if (!selection)
                        return;
                    if (selection.type == "Session") {
                        session_selector.emit("selected", selection);
                    }
                    else if (selection.type == "Container") {
                        session_selector.selection = selection;
                        session_selector.setInput(selection);
                    }
                }
            });
            _this.setContentProvider(session_content_provider);
            _this.setLabelProvider({
                label: new SessionLabelProvider(true),
                getText: function (element, columnIndex, text) {
                    if (columnIndex == SessionSelector.COL_NAME)
                        return this.label.getText(element);
                    if (element.type === "Session") {
                        if (columnIndex == SessionSelector.COL_TIME)
                            return new DateFormat(element.date).toDateString();
                        else if (columnIndex == SessionSelector.COL_COMMENTS) {
                            var session_1 = new Session(element.path + "/" + element.name, true);
                            session_1.setOn();
                            session_1.modelAvailable().then(function () {
                                text(session_1.getUserComment());
                            });
                            return "";
                        }
                        else if (columnIndex == SessionSelector.COL_STATUS) {
                            var session = new Session(element.path + "/" + element.name, true);
                            var statusProvider_1 = new GStatusQueryProvider();
                            statusProvider_1.on("changed", function (event) {
                                if (event.itemsChanged) {
                                    session_selector.statusMap[element.name] = statusProvider_1.getActiveStatus();
                                    text(statusProvider_1.getActiveStatus());
                                }
                            });
                            statusProvider_1.setSession(session);
                            return "";
                        }
                    }
                    return "";
                },
                getCssClass: function (element, columnIndex, text) {
                    if (columnIndex == SessionSelector.COL_NAME)
                        return "session_table_runname";
                    if (columnIndex == SessionSelector.COL_TIME)
                        return "session_table_starttime";
                    if (columnIndex == SessionSelector.COL_COMMENTS)
                        return "session_table_comment";
                    if (columnIndex == SessionSelector.COL_STATUS)
                        return "session_table_status";
                    return "";
                },
                getIcon: function (element, columnIndex, icon) {
                    if (element.type === "Session" && columnIndex == SessionSelector.COL_NAME)
                        return "images/session16x16.png";
                    else if (element.type === "Container" && columnIndex == SessionSelector.COL_NAME)
                        return "images/folder16x16.png";
                    return null;
                }
            });
            return _this;
        }
        SessionSelector.prototype.toggleMenu = function () {
            if (this.filter_menu) {
                this.filter_menu.remove();
                this.filter_menu = undefined;
                return;
            }
            var menu_model = $("<ul>");
            for (var i = 0; i < this.filter_tab.length; i++) {
                var li = $("<li>").text(this.filter_tab[i]).appendTo(menu_model);
                if (this.filter_value == this.filter_tab[i])
                    li.css("font-weight", "bold");
            }
            menu_model.css("position", "absolute");
            menu_model.css("z-index", "100");
            var menu = this;
            this.filter_menu = menu_model.menu({
                select: function (event, ui) {
                    if (ui.item) {
                        menu.toggleFilter(ui.item.text());
                        menu.toggleMenu();
                    }
                }
            });
            this.filter_menu.appendTo($("body"));
            this.filter_menu.position({ my: "left top", at: "left bottom", of: $(this.filter_button),
                collision: "fit" });
        };
        SessionSelector.prototype.toggleFilter = function (text) {
            var _this = this;
            if (this.getFilter() && this.filter_value == text) {
                this.setFilter(null);
                this.filter_value = null;
                return;
            }
            this.filter_value = text;
            this.setFilter(function (element) {
                if (element.path == "")
                    return true;
                return _this.statusMap[element.name] == text;
            });
        };
        SessionSelector.prototype.update = function () {
            _super.prototype.update.call(this);
        };
        SessionSelector.prototype.getSelection = function () {
            return this.selection;
        };
        return SessionSelector;
    }(TableBuilder_1.TableBuilder));
    SessionSelector.COL_NAME = 0;
    SessionSelector.COL_TIME = 1;
    SessionSelector.COL_STATUS = 2;
    SessionSelector.COL_COMMENTS = 3;
    return SessionSelector;
});
