var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "view/util/OrnamentProviders", "jrptlib/Properties!APPMSG"], function (require, exports, OrnamentProviders_1, APPMSG) {
    "use strict";
    exports.__esModule = true;
    var LegendOptions = (function () {
        function LegendOptions(_a) {
            var _b = _a.showTitle, showTitle = _b === void 0 ? false : _b, _c = _a.allowDoubleClick, allowDoubleClick = _c === void 0 ? true : _c;
            this.showTitle = showTitle;
            this.allowDoubleClick = allowDoubleClick;
        }
        return LegendOptions;
    }());
    exports.LegendOptions = LegendOptions;
    var MAX_LENGTH = 72;
    function short(str) {
        return str.length < MAX_LENGTH ? str : str.substring(0, MAX_LENGTH);
    }
    function long(str) {
        return str.length < MAX_LENGTH ? null : str;
    }
    var Legend = (function () {
        function Legend(provider, options, pager, nameFilter) {
            this.provider = provider;
            this.options = options;
            this.pager = pager;
            this.nameFilter = nameFilter;
        }
        Legend.create = function (provider, ornamentProvider, options, pager, nameFilter) {
            if (OrnamentProviders_1.isColorProvider(ornamentProvider))
                return new ColorLegend(provider, ornamentProvider, options, pager, nameFilter);
            if (OrnamentProviders_1.isStrokeProvider(ornamentProvider))
                return new StrokeLegend(provider, ornamentProvider, options, pager, nameFilter);
            if (OrnamentProviders_1.isShadeProvider(ornamentProvider))
                return new ShadeLegend(provider, ornamentProvider, options, pager, nameFilter);
            return new RegularLegend(provider, options, pager, nameFilter);
        };
        Legend.prototype.getProvider = function () {
            return this.provider;
        };
        Legend.prototype.adjustBannerVisibility = function () {
            var show = (this.nameFilter && this.nameFilter.isVisible()) ||
                (this.pager && this.pager.isVisible());
            var b = $(this.banner.node());
            if (show)
                b.show();
            else
                b.hide();
        };
        Legend.prototype.renderContents = function (parent) {
            var _this = this;
            var wrapper = parent.append("div")
                .classed("legend-wrapper", true);
            if (this.nameFilter || this.pager) {
                this.banner = wrapper.append("div")
                    .classed("legend-banner", true);
                if (this.nameFilter) {
                    this.nameFilter.renderContents(this.banner, function () { return _this.adjustBannerVisibility(); });
                }
                if (this.pager) {
                    this.pager.renderContents(this.banner, function () { return _this.adjustBannerVisibility(); });
                }
            }
            this.legend = wrapper.append("div")
                .classed("legend", true);
            if (this.options.showTitle) {
                this.legend.append("header").text(this.provider.name());
            }
            var body = this.legend.append("div")
                .classed("legend-body", true);
            body.append("ul");
            var footer = this.legend.append("footer");
            footer.append("a").text(APPMSG.SelectAllButton_label)
                .attr("id", "selectAll")
                .style("visibility", "hidden")
                .attr("href", "#")
                .attr("class", "btn")
                .on("click", function () { return _this.selectAll(); });
            this.renderItems();
            this.provider.on("changed", function (event) { return _this.processChange(event); });
        };
        Legend.prototype.processChange = function (event) {
            if (event.sourceItemsChanged) {
                this.legend.select("div").select("ul").selectAll("li").remove();
                this.renderItems();
            }
            else if (event.selectionChanged) {
                this.selectionChanged();
            }
        };
        Legend.prototype.renderItems = function () {
            var _this = this;
            var source = this.provider.source();
            this.items = this.legend.select("ul").selectAll("li")
                .data(source.items(), source.key());
            var a = this.items.enter()
                .append("li")
                .append("a")
                .attr("class", "btn")
                .attr("href", "#")
                .on("click", function (item, index) { _this.switchSelection(index); d3.event.preventDefault(); });
            if (this.options.allowDoubleClick) {
                a.on("dblclick", function (item, index) { _this.selectOnly(index); d3.event.preventDefault(); });
            }
            this.renderItemVisual(a);
            a.append("span").each(function (item, index) {
                var a = d3.select(this);
                var label = source.label(item, index);
                a.attr("title", long(label))
                    .text(short(label));
            });
        };
        Legend.prototype.switchSelection = function (index) {
            var selected = this.provider.isItemSelected(index);
            this.provider.selectItem(index, !selected);
        };
        Legend.prototype.selectOnly = function (index) {
            this.provider.selectItems([index]);
        };
        Legend.prototype.selectAll = function () {
            this.provider.selectItems(undefined);
        };
        Legend.prototype.selectionChanged = function () {
            this.updateItemVisual(this.items);
            this.legend.select("footer").select("#selectAll")
                .style("visibility", this.provider.isAllItemsSelected() ? "hidden" : "visible");
        };
        Legend.prototype.highlight = function (index) {
            this.items.transition().duration(100).style("opacity", function (item, idx) { return idx != index ? .18 : 1; });
        };
        Legend.prototype.highlightMultiple = function (indices) {
            this.items.transition().duration(100).style("opacity", function (item, index) { return indices.indexOf(index) == -1 ? .18 : 1; });
        };
        Legend.prototype.clearHighlight = function () {
            this.items.transition().style("opacity", 1);
        };
        return Legend;
    }());
    exports.Legend = Legend;
    var GOrnamentedLegend = (function (_super) {
        __extends(GOrnamentedLegend, _super);
        function GOrnamentedLegend(provider, ornamentProvider, options, pager, nameFilter) {
            var _this = _super.call(this, provider, options, pager, nameFilter) || this;
            _this.ornamentProvider = ornamentProvider;
            return _this;
        }
        return GOrnamentedLegend;
    }(Legend));
    function colorFill(legendProvider, colorProvider) {
        return function (item, index) {
            return legendProvider.isItemSelected(index) ? colorProvider.color(index) : "none";
        };
    }
    function colorStroke(colorProvider) {
        return function (item, index) {
            return colorProvider.color(index);
        };
    }
    var ColorLegend = (function (_super) {
        __extends(ColorLegend, _super);
        function ColorLegend() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ColorLegend.prototype.renderItemVisual = function (a) {
            a.append("svg")
                .attr("height", 15)
                .attr("width", 25)
                .append("rect")
                .attr("height", 15)
                .attr("width", 25)
                .attr("fill", colorFill(this.provider, this.ornamentProvider))
                .attr("stroke", colorStroke(this.ornamentProvider));
        };
        ColorLegend.prototype.updateItemVisual = function (a) {
            a.select("svg").select("rect").transition().attr("fill", colorFill(this.provider, this.ornamentProvider));
        };
        ColorLegend.prototype.getHighlightColor = function (item) {
            var rect = item.select("svg").select("rect");
            return rect.attr("fill");
        };
        return ColorLegend;
    }(GOrnamentedLegend));
    function rectStroke(provider) {
        return function (item, index) {
            return provider.isItemSelected(index) ? "#444444" : "#bbbbbb";
        };
    }
    var StrokeLegend = (function (_super) {
        __extends(StrokeLegend, _super);
        function StrokeLegend(provider, strokeProvider, options, pager, nameFilter) {
            return _super.call(this, provider, strokeProvider, options, pager, nameFilter) || this;
        }
        StrokeLegend.prototype.renderItemVisual = function (a) {
            var _this = this;
            var svg = a.append("svg")
                .attr("height", 15)
                .attr("width", 65);
            svg.append("rect")
                .attr("height", 15)
                .attr("width", 65)
                .attr("fill", "none")
                .attr("stroke", rectStroke(this.provider));
            svg.append("path")
                .attr("d", "M 0 7.5 H 65")
                .attr("stroke", "black")
                .attr("stroke-dasharray", function (item, index) { return _this.ornamentProvider.stroke(index); });
        };
        StrokeLegend.prototype.updateItemVisual = function (a) {
            a.select("svg").select("rect").transition().attr("stroke", rectStroke(this.provider));
        };
        return StrokeLegend;
    }(GOrnamentedLegend));
    function shadeFill(legendProvider, shadeProvider) {
        return function (item, index) {
            return legendProvider.isItemSelected(index) ? shadeProvider.shade("#333333", index) : "none";
        };
    }
    var ShadeLegend = (function (_super) {
        __extends(ShadeLegend, _super);
        function ShadeLegend(provider, shadeProvider, options, pager, nameFilter) {
            return _super.call(this, provider, shadeProvider, options, pager, nameFilter) || this;
        }
        ShadeLegend.prototype.renderItemVisual = function (a) {
            a.append("svg")
                .attr("height", 15)
                .attr("width", 25)
                .append("rect")
                .attr("height", 15)
                .attr("width", 25)
                .attr("fill", shadeFill(this.provider, this.ornamentProvider))
                .attr("stroke", "black");
        };
        ShadeLegend.prototype.updateItemVisual = function (a) {
            a.select("svg").select("rect").transition().attr("fill", shadeFill(this.provider, this.ornamentProvider));
        };
        return ShadeLegend;
    }(GOrnamentedLegend));
    function regularFill(provider) {
        return function (item, index) {
            return provider.isItemSelected(index) ? "gray" : "none";
        };
    }
    var RegularLegend = (function (_super) {
        __extends(RegularLegend, _super);
        function RegularLegend(provider, options, pager, nameFilter) {
            return _super.call(this, provider, options, pager, nameFilter) || this;
        }
        RegularLegend.prototype.renderItemVisual = function (a) {
            a.append("svg")
                .attr("height", 15)
                .attr("width", 25)
                .append("rect")
                .attr("height", 15)
                .attr("width", 25)
                .attr("fill", regularFill(this.provider))
                .attr("stroke", "black");
        };
        RegularLegend.prototype.updateItemVisual = function (a) {
            a.select("svg").select("rect").transition().attr("fill", regularFill(this.provider));
        };
        return RegularLegend;
    }(Legend));
});
