var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "view/comps/Legend", "view/comps/Pager", "view/comps/NameFilter", "view/util/FilterableDimensionProvider", "view/util/FilterableMatrixProvider", "view/util/OrnamentProviders", "view/util/DimensionUtil"], function (require, exports, Legend_1, Pager_1, NameFilter_1, FilterableDimensionProvider_1, FilterableMatrixProvider_1, OrnamentProviders_1, du) {
    "use strict";
    exports.__esModule = true;
    var MultiLegend = (function () {
        function MultiLegend(provider, options, ornamentKinds, scaleOnSelection) {
            var nameFilter;
            var pager;
            if (options.usePaging() || options.useFilter() || options.useSorting()) {
                nameFilter = [];
                pager = [];
                var filterDims = [];
                for (var i = 0; i < options.paging.length; i++) {
                    if (!options.legends[i]) {
                        if (!this.external)
                            this.external = [];
                        this.external[i] = {};
                    }
                    var dim = provider.dimension(i);
                    var filterDim = void 0;
                    if (options.sortings[i]) {
                        dim = filterDim = new FilterableDimensionProvider_1.SortingDimensionProvider(dim, i);
                    }
                    if (options.filters[i]) {
                        var nameFilterDim = new FilterableDimensionProvider_1.NameFilterDimensionProvider(dim);
                        nameFilter[i] = new NameFilter_1.NameFilter(nameFilterDim, options.filters[i]);
                        if (!options.legends[i]) {
                            this.external[i].nameFilter = nameFilter[i];
                        }
                        dim = filterDim = nameFilterDim;
                    }
                    if (options.paging[i]) {
                        var pagingDim = void 0;
                        if (options.legends[i]) {
                            pagingDim = new FilterableDimensionProvider_1.PagingDimensionProvider(dim, options.paging[i]);
                            pager[i] = new Pager_1.Pager(pagingDim);
                        }
                        else {
                            pagingDim = this.external[i].windower = new FilterableDimensionProvider_1.WindowingDimensionProvider(dim, options.paging[i]);
                        }
                        dim = filterDim = pagingDim;
                    }
                    filterDims[i] = filterDim;
                }
                provider = this.createFilterableMatrix(provider, filterDims, scaleOnSelection);
            }
            var ornamentMatrix = new OrnamentProviders_1.OrnamentMatrix(provider, ornamentKinds);
            var legendCount = options.legendCount();
            if (legendCount > 0) {
                this.legend = [];
                var filteredDims = [];
                for (var i = 0; i < options.legends.length; i++) {
                    if (options.legends[i]) {
                        filteredDims[i] = new FilterableDimensionProvider_1.FilterableElementDimensionProvider(provider.dimension(i));
                        this.legend[i] = Legend_1.Legend.create(filteredDims[i], ornamentMatrix.ornament(i), new Legend_1.LegendOptions({ showTitle: legendCount > 1 }), pager ? pager[i] : null, nameFilter ? nameFilter[i] : null);
                    }
                }
                var filter = this.createFilterableMatrix(provider, filteredDims, scaleOnSelection);
                this.provider = filter;
                this.filteredOrnamentMatrix = filter.filteredOrnament(ornamentMatrix);
            }
            else {
                this.provider = provider;
                this.filteredOrnamentMatrix = ornamentMatrix;
            }
        }
        MultiLegend.prototype.getProvider = function () {
            return this.provider;
        };
        MultiLegend.prototype.getOrnamentMatrix = function () {
            return this.filteredOrnamentMatrix;
        };
        MultiLegend.prototype.getExternalLegend = function (dimension) {
            return this.external ? this.external[dimension] : null;
        };
        MultiLegend.prototype.renderContents = function (parentDiv) {
            for (var i = 0; i < this.provider.dimensionsCount(); i++) {
                var legend = this.legend ? this.legend[i] : null;
                if (legend) {
                    legend.renderContents(parentDiv);
                }
            }
        };
        MultiLegend.prototype.getLegendProvider = function (dim) {
            if (this.legend && this.legend[dim])
                return this.legend[dim].getProvider();
            return null;
        };
        MultiLegend.prototype.highlightIndex = function (index) {
            if (!this.legend)
                return;
            var n = this.provider.dimensionsCount();
            for (var i = 0; i < 3; i++) {
                var lp = this.getLegendProvider(i);
                if (lp) {
                    if (index[i] != -1) {
                        var legendIndex = lp.sourceIndex(index[i]);
                        this.legend[i].highlight(legendIndex);
                    }
                    else {
                        this.legend[i].clearHighlight();
                    }
                }
            }
        };
        MultiLegend.prototype.highlightIndices = function (indices) {
            if (!this.legend)
                return;
            var n = this.provider.dimensionsCount();
            var _loop_1 = function (i) {
                var lp = this_1.getLegendProvider(i);
                if (lp) {
                    var _indices = indices(i);
                    _indices = _indices.map(function (index) { return lp.sourceIndex(index); });
                    this_1.legend[i].highlightMultiple(_indices);
                }
            };
            var this_1 = this;
            for (var i = 0; i < n; i++) {
                _loop_1(i);
            }
        };
        MultiLegend.prototype.clearHighlight = function () {
            if (!this.legend)
                return;
            for (var i = 0; i < 2; i++) {
                if (this.legend[i])
                    this.legend[i].clearHighlight();
            }
        };
        return MultiLegend;
    }());
    exports.MultiLegend = MultiLegend;
    var TabularMultiLegend = (function (_super) {
        __extends(TabularMultiLegend, _super);
        function TabularMultiLegend() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TabularMultiLegend.prototype.createFilterableMatrix = function (provider, filterDims, scaleOnSelection) {
            return new FilterableMatrixProvider_1.FilterableTabularProvider(provider, filterDims, scaleOnSelection);
        };
        return TabularMultiLegend;
    }(MultiLegend));
    exports.TabularMultiLegend = TabularMultiLegend;
    var LinesMultiLegend = (function (_super) {
        __extends(LinesMultiLegend, _super);
        function LinesMultiLegend() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LinesMultiLegend.prototype.createFilterableMatrix = function (provider, filterDims, scaleOnSelection) {
            return du.isVariableProvider(provider) ?
                new FilterableMatrixProvider_1.FilterableVariableLinesProvider(provider, filterDims, scaleOnSelection) :
                new FilterableMatrixProvider_1.FilterableLinesProvider(provider, filterDims, scaleOnSelection);
        };
        return LinesMultiLegend;
    }(MultiLegend));
    exports.LinesMultiLegend = LinesMultiLegend;
});
