define(["require", "exports", "jrptlib/Nls"], function (require, exports, NLS) {
    "use strict";
    exports.__esModule = true;
    var NameFilter = (function () {
        function NameFilter(provider, minItems) {
            this.visible = true;
            this.provider = provider;
            this.minItems = minItems;
        }
        NameFilter.prototype.processChange = function (event) {
            if (event.filterChanged) {
                this.updateCount();
            }
        };
        NameFilter.prototype.renderContents = function (parent, visiblityListener) {
            var _this = this;
            this.visiblityListener = visiblityListener;
            var filter = this;
            this.div = parent.append("div")
                .classed("name-filter-wrapper", true);
            this.div.append("a")
                .classed("btn", true)
                .attr("href", "#")
                .attr("title", "Toggle Filter")
                .attr("role", "button")
                .on("click", function () { return _this.toggleFilter(); })
                .append("img")
                .attr("src", "images/filter-legend.png");
            var panel = this.div.append("div")
                .classed("name-filter", true)
                .style("display", "none");
            panel.append("input")
                .attr("placeholder", "Filter elements")
                .attr("size", 40)
                .on("input", function () {
                filter.provider.setFilter(this.value);
            })
                .on("keypress", function () {
                if (d3.event.keyCode == 27)
                    _this.toggleFilter();
            });
            panel.append("span");
            this.updateCount();
            this.provider.on("changed", function (event) { return _this.processChange(event); });
        };
        NameFilter.prototype.toggleFilter = function () {
            d3.event.preventDefault();
            var filter = this.div.select(".name-filter");
            var node = $(filter.node());
            node.toggle();
            if (!node.is(":visible")) {
                this.provider.setFilter("");
                filter.select("input").node().value = "";
            }
            else {
                filter.select("input").node().focus();
            }
        };
        NameFilter.prototype.updateCount = function () {
            var count = this.provider.size();
            this.div.select(".name-filter").select("span")
                .text(NLS.bind(this.provider.filter() ? "{0} elements matched." : "{0} elements.", count));
            var sourceCount = this.provider.source().size();
            this.setVisible(sourceCount > this.minItems);
        };
        NameFilter.prototype.setVisible = function (visible) {
            if (this.visible == visible)
                return;
            var div = $(this.div.node());
            if (visible)
                div.show();
            else
                div.hide();
            this.visible = visible;
            if (this.visiblityListener)
                this.visiblityListener(visible);
        };
        NameFilter.prototype.isVisible = function () {
            return this.visible;
        };
        return NameFilter;
    }());
    exports.NameFilter = NameFilter;
});
