define(["require", "exports"], function (require, exports) {
    "use strict";
    exports.__esModule = true;
    var PAGER_SIZE = 9;
    function generatePages(count, current, size) {
        if (count <= size) {
            return d3.range(0, count);
        }
        else {
            var cell = size - 4, start_1 = Math.floor(current - cell / 2 + 1);
            if (start_1 <= 2) {
                return d3.range(0, size - 2).concat([-1, count - 1]);
            }
            else if (start_1 + cell >= count - 2) {
                return [0, -1].concat(d3.range(count - size + 2, count));
            }
            else {
                return [0, -1].concat(d3.range(start_1, start_1 + cell)).concat([-2, count - 1]);
            }
        }
    }
    function pageText(page) {
        return page < 0 ? "..." : "" + (page + 1);
    }
    function pageTitle(page) {
        return page < 0 ? null : "Page " + (page + 1);
    }
    var Pager = (function () {
        function Pager(provider) {
            this.visible = true;
            this.provider = provider;
        }
        Pager.prototype.processChange = function (event) {
            if (event.pageCountChanged || event.pageChanged) {
                this.updatePager();
            }
        };
        Pager.prototype.renderContents = function (parent, visiblityListener) {
            var _this = this;
            this.visiblityListener = visiblityListener;
            this.ul = parent.append("ul")
                .classed("pager", true);
            this.updatePager();
            this.provider.on("changed", function (event) { return _this.processChange(event); });
        };
        Pager.prototype.updatePager = function () {
            var _this = this;
            var count = this.provider.pageCount();
            this.setVisible(count > 1);
            if (count > 1) {
                $(this.ul.node()).show();
                var current_1 = this.provider.page();
                var ul = this.ul;
                var pages = generatePages(this.provider.pageCount(), current_1, PAGER_SIZE);
                ul.selectAll("li").remove();
                var sel = ul.selectAll("li").data(pages);
                sel.enter()
                    .append("li")
                    .append("a")
                    .classed("selected", function (i) { return i == current_1; })
                    .classed("break", function (i) { return i < 0; })
                    .classed("btn", function (p) { return p >= 0 && p != current_1; })
                    .attr("role", function (p) { return p < 0 || p == current_1 ? null : "button"; })
                    .attr("alt", pageTitle)
                    .attr("href", function (p) { return p < 0 || p == current_1 ? null : "#"; })
                    .text(pageText)
                    .on("click", function (p) { d3.event.preventDefault(); if (p >= 0)
                    _this.provider.setPage(p); });
            }
        };
        Pager.prototype.setVisible = function (visible) {
            if (this.visible == visible)
                return;
            var ul = $(this.ul.node());
            if (visible)
                ul.show();
            else
                ul.hide();
            this.visible = visible;
            if (this.visiblityListener)
                this.visiblityListener(visible);
        };
        Pager.prototype.isVisible = function () {
            return this.visible;
        };
        return Pager;
    }());
    exports.Pager = Pager;
});
