define(["require", "exports"], function (require, exports) {
    "use strict";
    exports.__esModule = true;
    var MARGIN = 1;
    var BAR_HEIGHT = 7;
    var TOTAL_HEIGHT = BAR_HEIGHT + MARGIN;
    var CURSOR = "ew-resize";
    var WIDTH_PER_BAR = 30;
    var Slider = (function () {
        function Slider(provider) {
            this.visible = true;
            this.provider = provider;
        }
        Slider.prototype.processChange = function (event) {
            if (event.startIndexChanged || event.highestStartIndexChanged) {
                this.updateSlider();
            }
        };
        Slider.prototype.getHeight = function () {
            return this.visible ? TOTAL_HEIGHT : 0;
        };
        Slider.prototype.renderContents = function (parent, size, minCellWidth, visiblityListener) {
            var _this = this;
            this.visiblityListener = visiblityListener;
            this.width = size[0];
            this.minCellWidth = minCellWidth;
            this.provider.setWindowSize(Math.ceil(this.width / minCellWidth));
            this.domBrush = parent.append("g")
                .attr("class", "slider")
                .attr("transform", "translate(0, " + size[1] + ")")
                .style("pointer-events", "all");
            this.domBrush.append("rect")
                .classed("background", true)
                .style("visibility", "hidden")
                .style("cursor", CURSOR)
                .attr("width", size[0])
                .attr("height", TOTAL_HEIGHT)
                .on("mousedown", function () { return _this.handlePage(); });
            var extent = this.domBrush.append("rect")
                .classed("extent", true)
                .style("cursor", CURSOR)
                .attr("y", MARGIN)
                .attr("height", BAR_HEIGHT)
                .attr("rx", 5)
                .attr("ry", 5)
                .on("mousedown", function () { return _this.handleMove(); });
            this.updateExtent(extent);
            this.provider.on("changed", function (event) { return _this.processChange(event); });
        };
        Slider.prototype.handlePage = function () {
            var kc = d3.mouse(this.domBrush.node());
            var extent = this.domBrush.select(".extent");
            var x = +extent.attr("x");
            if (kc[0] < x) {
                var idx = Math.max(this.provider.startIndex() - this.provider.windowSize(), 0);
                this.provider.setStartIndex(idx);
            }
            else if (kc[0] > x + +extent.attr("width")) {
                var idx = Math.min(this.provider.startIndex() + this.provider.windowSize(), this.provider.highestStartIndex());
                this.provider.setStartIndex(idx);
            }
        };
        Slider.prototype.handleMove = function () {
            var _this = this;
            d3.event.preventDefault();
            var kc = d3.mouse(this.domBrush.node());
            var background = this.domBrush.select(".background").style("pointer-events", "none");
            var extent = this.domBrush.select(".extent");
            var x = +extent.attr("x");
            var offset = kc[0] - x;
            var w = d3.select(window);
            var scale = d3.scale.linear()
                .domain([0, this.provider.source().size() - 1])
                .range([0, this.width]);
            d3.select("body").style("cursor", CURSOR);
            w.on("mousemove", function () {
                var kc = d3.mouse(_this.domBrush.node());
                var pos = kc[0] - offset;
                var idx = Math.min(_this.provider.highestStartIndex(), Math.max(0, Math.round(scale.invert(pos))));
                _this.provider.setStartIndex(idx);
            });
            w.on("mouseup", function () {
                w.on("mousemove", null);
                background.style("pointer-events", "all");
                d3.select("body").style("cursor", null);
            });
        };
        Slider.prototype.setWidth = function (width) {
            if (this.width == width)
                return;
            this.width = width;
            this.domBrush.select(".background")
                .attr("width", width);
            if (!this.provider.setWindowSize(Math.ceil(this.width / this.minCellWidth))) {
                this.updateSlider();
            }
        };
        Slider.prototype.setMinCellWidth = function (width) {
            if (this.minCellWidth == width)
                return;
            this.minCellWidth = width;
            this.provider.setWindowSize(Math.ceil(this.width / this.minCellWidth));
        };
        Slider.prototype.setHeight = function (height) {
            this.domBrush.attr("transform", "translate(0, " + (height + MARGIN) + ")");
        };
        Slider.prototype.updateSlider = function () {
            this.setVisible(this.provider.highestStartIndex() > 0);
            this.updateExtent(this.domBrush.select(".extent"));
        };
        Slider.prototype.updateExtent = function (extent) {
            var scale = d3.scale.linear()
                .domain([0, this.provider.source().size()])
                .range([0, this.width]);
            extent
                .attr("x", scale(this.provider.startIndex()))
                .attr("width", scale(this.provider.size()));
        };
        Slider.prototype.setVisible = function (visible) {
            if (this.visible == visible)
                return;
            var ul = $(this.domBrush.node());
            if (visible)
                ul.show();
            else
                ul.hide();
            this.visible = visible;
            if (this.visiblityListener)
                this.visiblityListener(visible);
        };
        Slider.prototype.isVisible = function () {
            return this.visible;
        };
        return Slider;
    }());
    exports.Slider = Slider;
});
