var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "jrptlib/Evented", "view/util/ItemFilter", "view/util/OrnamentProviders", "view/util/Reordering"], function (require, exports, Evented, ItemFilter_1, OrnamentProviders_1, Reordering) {
    "use strict";
    exports.__esModule = true;
    function isFilter(provider) {
        return "itemsChanged" in provider;
    }
    var FilterableDimensionProvider = (function (_super) {
        __extends(FilterableDimensionProvider, _super);
        function FilterableDimensionProvider(provider) {
            var _this = _super.call(this) || this;
            _this.provider = provider;
            return _this;
        }
        FilterableDimensionProvider.prototype.setOwner = function (owner) {
            var _this = this;
            if (this.owner)
                return false;
            this.owner = owner;
            if (isFilter(this.provider)) {
                if (this.provider.setOwner(owner)) {
                    this.previous = this.provider;
                    this.previous.setSelectionChangedHandler(function () {
                        _this.sourceItemsChanged();
                        _this.selectionChangedHandler();
                    });
                }
            }
            return true;
        };
        FilterableDimensionProvider.prototype.getOwner = function () {
            return this.owner;
        };
        FilterableDimensionProvider.prototype.setSelectionChangedHandler = function (handler) {
            this.selectionChangedHandler = handler;
        };
        FilterableDimensionProvider.prototype.itemsChanged = function () {
            if (this.previous)
                this.previous.itemsChanged();
            this.sourceItemsChanged();
        };
        FilterableDimensionProvider.prototype.source = function () {
            return this.provider;
        };
        FilterableDimensionProvider.prototype.name = function () {
            return this.provider.name();
        };
        FilterableDimensionProvider.prototype.key = function () {
            return this.provider.key();
        };
        FilterableDimensionProvider.prototype.sourceIndex = function (index) {
            var sidx = this.localSourceIndex(index);
            return this.previous ? this.previous.sourceIndex(sidx) : sidx;
        };
        FilterableDimensionProvider.prototype.sourceIndices = function (indices) {
            var sidx = this.localSourceIndices(indices);
            return this.previous ? this.previous.sourceIndices(sidx) : sidx;
        };
        FilterableDimensionProvider.prototype.label = function (item, index) {
            return this.provider.label(item, this.sourceIndex(index));
        };
        FilterableDimensionProvider.prototype.ornament = function (ornament) {
            var _this = this;
            if (OrnamentProviders_1.isColorProvider(ornament))
                return {
                    color: function (index) {
                        return ornament.color(_this.sourceIndex(index));
                    }
                };
            if (OrnamentProviders_1.isStrokeProvider(ornament))
                return {
                    stroke: function (index) {
                        return ornament.stroke(_this.sourceIndex(index));
                    }
                };
            if (OrnamentProviders_1.isShadeProvider(ornament))
                return {
                    shade: function (color, index) {
                        return ornament.shade(color, _this.sourceIndex(index));
                    }
                };
            return null;
        };
        return FilterableDimensionProvider;
    }(Evented));
    exports.FilterableDimensionProvider = FilterableDimensionProvider;
    var ItemFilterBasedDimensionProvider = (function (_super) {
        __extends(ItemFilterBasedDimensionProvider, _super);
        function ItemFilterBasedDimensionProvider(provider) {
            var _this = _super.call(this, provider) || this;
            _this._filter = new ItemFilter_1.ItemFilter(_this.provider.size());
            return _this;
        }
        ItemFilterBasedDimensionProvider.prototype.size = function () {
            return this._filter.size();
        };
        ItemFilterBasedDimensionProvider.prototype.items = function () {
            return this._filter.apply(this.provider.items());
        };
        ItemFilterBasedDimensionProvider.prototype.key = function () {
            var f = this.provider.key();
            if (f)
                return function (item) { return f(item.object); };
            return function (item) { return item.index.toString(); };
        };
        ItemFilterBasedDimensionProvider.prototype.label = function (item, index) {
            return this.provider.label(item.object, item.index);
        };
        ItemFilterBasedDimensionProvider.prototype.localSourceIndex = function (index) {
            return this._filter.index(index);
        };
        ItemFilterBasedDimensionProvider.prototype.localSourceIndices = function (indices) {
            return this._filter.unfilteredIndices(indices);
        };
        return ItemFilterBasedDimensionProvider;
    }(FilterableDimensionProvider));
    var FilterableElementDimensionProvider = (function (_super) {
        __extends(FilterableElementDimensionProvider, _super);
        function FilterableElementDimensionProvider(provider) {
            return _super.call(this, provider) || this;
        }
        FilterableElementDimensionProvider.prototype.isItemSelected = function (index) {
            return this._filter.contains(index);
        };
        FilterableElementDimensionProvider.prototype.isAllItemsSelected = function () {
            return this._filter.isFull();
        };
        FilterableElementDimensionProvider.prototype.selectItem = function (index, selected) {
            if (selected)
                this._filter.add(index);
            else
                this._filter.remove(index);
            this.emit("changed", {
                selectionChanged: true
            });
            this.selectionChangedHandler();
        };
        FilterableElementDimensionProvider.prototype.selectItems = function (indices) {
            this._filter.setSelected(indices);
            this.emit("changed", {
                selectionChanged: true
            });
            this.selectionChangedHandler();
        };
        FilterableElementDimensionProvider.prototype.sourceItemsChanged = function () {
            this._filter = new ItemFilter_1.ItemFilter(this.provider.size());
            this.emit("changed", {
                sourceItemsChanged: true,
                selectionChanged: true
            });
        };
        return FilterableElementDimensionProvider;
    }(ItemFilterBasedDimensionProvider));
    exports.FilterableElementDimensionProvider = FilterableElementDimensionProvider;
    function escapeRegexp(str) {
        return str.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1");
    }
    function compile(str) {
        return str.split(" ").filter(function (s) { return s.length != 0; }).map(function (s) { return new RegExp(escapeRegexp(s.trim()), "i"); });
    }
    var NameFilterDimensionProvider = (function (_super) {
        __extends(NameFilterDimensionProvider, _super);
        function NameFilterDimensionProvider(provider) {
            return _super.call(this, provider) || this;
        }
        NameFilterDimensionProvider.prototype.exclude = function (pattern, name) {
            return !pattern.some(function (p) { return p.test(name); });
        };
        NameFilterDimensionProvider.prototype.update = function () {
            this._filter = new ItemFilter_1.ItemFilter(this.provider.size());
            if (this._pattern) {
                var compiled = compile(this._pattern);
                var items = this.provider.items();
                for (var i = 0; i < items.length; i++) {
                    if (this.exclude(compiled, this.provider.label(items[i], i))) {
                        this._filter.remove(i);
                    }
                }
            }
            this.emit("changed", {
                filterChanged: true
            });
        };
        NameFilterDimensionProvider.prototype.filter = function () {
            return this._pattern;
        };
        NameFilterDimensionProvider.prototype.setFilter = function (filter) {
            this._pattern = filter;
            this.update();
            this.selectionChangedHandler();
        };
        NameFilterDimensionProvider.prototype.sourceItemsChanged = function () {
            this.update();
        };
        return NameFilterDimensionProvider;
    }(ItemFilterBasedDimensionProvider));
    exports.NameFilterDimensionProvider = NameFilterDimensionProvider;
    function divUp(D, d) {
        return ((D + d - 1) / d) | 0;
    }
    var PagingDimensionProvider = (function (_super) {
        __extends(PagingDimensionProvider, _super);
        function PagingDimensionProvider(provider, pageSize) {
            var _this = _super.call(this, provider) || this;
            _this._page = 0;
            _this._fullPages = 1;
            _this._remainder = 0;
            _this.pageSize = pageSize;
            _this.update(false);
            return _this;
        }
        PagingDimensionProvider.prototype.update = function (notify) {
            var oldPageCount = this.pageCount();
            this._fullPages = (this.provider.size() / this.pageSize) | 0;
            this._remainder = this.provider.size() % this.pageSize;
            var newPageCount = this.pageCount();
            var pageOver = this._page >= newPageCount;
            if (pageOver) {
                this._page = newPageCount - 1;
            }
            if (notify && (pageOver || oldPageCount != newPageCount)) {
                this.emit("changed", {
                    pageChanged: pageOver,
                    pageCountChanged: oldPageCount != newPageCount
                });
            }
        };
        PagingDimensionProvider.prototype.size = function () {
            if (this._page == this._fullPages) {
                return this._remainder;
            }
            return this.pageSize;
        };
        PagingDimensionProvider.prototype.items = function () {
            var start = this._page * this.pageSize;
            var items = this.provider.items();
            var end = Math.min(start + this.pageSize, items.length);
            return items.slice(start, end);
        };
        PagingDimensionProvider.prototype.localSourceIndex = function (index) {
            return this._page * this.pageSize + index;
        };
        PagingDimensionProvider.prototype.localSourceIndices = function (indices) {
            if (!indices && this.pageCount() == 1)
                return undefined;
            var start = this._page * this.pageSize;
            if (!indices) {
                var end = this._page == this._fullPages ? start + this._remainder : start + this.pageSize;
                return d3.range(start, end);
            }
            return indices.map(function (i) { return i + start; });
        };
        PagingDimensionProvider.prototype.pageCount = function () {
            return Math.max(1, this._fullPages + (this._remainder ? 1 : 0));
        };
        PagingDimensionProvider.prototype.page = function () {
            return this._page;
        };
        PagingDimensionProvider.prototype.setPage = function (page) {
            if (page < 0 || page >= this.pageCount())
                throw "page out of range";
            this._page = page;
            this.emit("changed", {
                pageChanged: true
            });
            this.selectionChangedHandler();
        };
        PagingDimensionProvider.prototype.sourceItemsChanged = function () {
            this.update(true);
        };
        return PagingDimensionProvider;
    }(FilterableDimensionProvider));
    exports.PagingDimensionProvider = PagingDimensionProvider;
    var WindowingDimensionProvider = (function (_super) {
        __extends(WindowingDimensionProvider, _super);
        function WindowingDimensionProvider(provider, windowSize) {
            var _this = _super.call(this, provider) || this;
            _this.sourceSize = 0;
            _this.start = 0;
            _this._windowSize = windowSize;
            _this.update(false);
            return _this;
        }
        WindowingDimensionProvider.prototype.update = function (notify) {
            var oldSize = this.sourceSize;
            this.sourceSize = this.provider.size();
            var newHighest = this.highestStartIndex();
            var startChanged = false;
            if (this.start > newHighest) {
                this.start = newHighest;
                startChanged = true;
            }
            if (notify && (oldSize != this.sourceSize || startChanged)) {
                this.emit("changed", {
                    startIndexChanged: startChanged,
                    highestStartIndexChanged: oldSize != this.sourceSize
                });
            }
        };
        WindowingDimensionProvider.prototype.size = function () {
            return Math.min(this.sourceSize, this._windowSize);
        };
        WindowingDimensionProvider.prototype.items = function () {
            var items = this.provider.items();
            var end = Math.min(this.start + this._windowSize, items.length);
            return items.slice(this.start, end);
        };
        WindowingDimensionProvider.prototype.localSourceIndex = function (index) {
            return index + this.start;
        };
        WindowingDimensionProvider.prototype.localSourceIndices = function (indices) {
            var _this = this;
            if (!indices) {
                var size = this.sourceSize;
                if (size <= this._windowSize)
                    return undefined;
                return d3.range(this.start, this.start + this._windowSize);
            }
            return indices.map(function (i) { return i + _this.start; });
        };
        WindowingDimensionProvider.prototype.startIndex = function () {
            return this.start;
        };
        WindowingDimensionProvider.prototype.lastIndex = function () {
            return Math.min(this.start + this._windowSize, this.sourceSize);
        };
        WindowingDimensionProvider.prototype.windowSize = function () {
            return this._windowSize;
        };
        WindowingDimensionProvider.prototype.highestStartIndex = function () {
            return Math.max(this.sourceSize - this._windowSize, 0);
        };
        WindowingDimensionProvider.prototype.setStartIndex = function (index) {
            if (index < 0 || index > this.highestStartIndex())
                throw "page out of range";
            if (index == this.start)
                return;
            this.start = index;
            this.emit("changed", {
                startIndexChanged: true
            });
            this.selectionChangedHandler(true);
        };
        WindowingDimensionProvider.prototype.setWindowSize = function (size) {
            if (size < 1)
                throw "windowSize out of range";
            if (size == this._windowSize)
                return;
            var oldStart = this.start;
            var oldHighest = this.highestStartIndex();
            this._windowSize = size;
            var newHighest = this.highestStartIndex();
            var startChanged = false;
            if (this.start > newHighest) {
                this.start = newHighest;
                startChanged = true;
            }
            if (oldHighest != newHighest || startChanged) {
                this.emit("changed", {
                    startIndexChanged: startChanged,
                    highestStartIndexChanged: oldHighest != newHighest
                });
                this.selectionChangedHandler(true);
                return true;
            }
            return false;
        };
        WindowingDimensionProvider.prototype.sourceItemsChanged = function () {
            this.update(true);
        };
        return WindowingDimensionProvider;
    }(FilterableDimensionProvider));
    exports.WindowingDimensionProvider = WindowingDimensionProvider;
    var SortMethod;
    (function (SortMethod) {
        SortMethod[SortMethod["ASCENT"] = 0] = "ASCENT";
        SortMethod[SortMethod["DESCENT"] = 1] = "DESCENT";
    })(SortMethod = exports.SortMethod || (exports.SortMethod = {}));
    var SortingDimensionProvider = (function (_super) {
        __extends(SortingDimensionProvider, _super);
        function SortingDimensionProvider(provider, dimIndex) {
            var _this = _super.call(this, provider) || this;
            _this.sortMethodLabel = SortMethod.ASCENT;
            _this.sortMethodValue = SortMethod.ASCENT;
            _this.dimIndex = dimIndex;
            _this.update();
            return _this;
        }
        SortingDimensionProvider.prototype.update = function () {
            var _this = this;
            var stringCompare = function (a, b, method) {
                var comp = a.localeCompare(b);
                return method == SortMethod.ASCENT ? comp : -comp;
            };
            if (!this.sortIndexes || this.sortIndexes.length == 0) {
                var label_1 = this.provider.label;
                var labels = this.provider.items().map(function (item, idx) { return { value: label_1(item, idx), index: idx }; });
                labels.sort(function (a, b) { return stringCompare(a.value, b.value, _this.sortMethodLabel); });
                this.reordering = new Reordering(labels.map(function (l) { return l.index; }));
            }
            else {
                var label_2 = this.provider.label;
                var values = this.provider.items().map(function (item, idx) {
                    var index = [];
                    for (var i = 0; i < _this.getOwner().dimensionsCount(); i++) {
                        if (i == _this.dimIndex) {
                            index[i] = _this.previous ? _this.previous.sourceIndex(idx) : idx;
                        }
                        else if (_this.sortIndexes[i]) {
                            index[i] = _this.sortIndexes[i];
                        }
                        else
                            index[i] = 0;
                    }
                    return { value: _this.getOwner().getSource().data(index),
                        index: idx, label: label_2(item, idx) };
                });
                values.sort(function (a, b) {
                    if (a.value === b.value) {
                        return stringCompare(a.label, b.label, _this.sortMethodLabel);
                    }
                    if ((isNaN(a.value) ? 0 : a.value) < (isNaN(b.value) ? 0 : b.value))
                        return (_this.sortMethodValue == SortMethod.ASCENT) ? -1 : 1;
                    return _this.sortMethodValue == SortMethod.ASCENT ? 1 : -1;
                });
                this.reordering = new Reordering(values.map(function (l) { return l.index; }));
            }
        };
        SortingDimensionProvider.prototype.setSortingValues = function (sortIndexes, method) {
            this.sortMethodValue = method;
            this.sortIndexes = sortIndexes;
            this.update();
            this.selectionChangedHandler(false);
        };
        SortingDimensionProvider.prototype.setSortingLabels = function (method) {
            this.sortMethodLabel = method;
            this.sortIndexes = null;
            this.update();
            this.selectionChangedHandler(false);
        };
        SortingDimensionProvider.prototype.size = function () {
            return this.provider.size();
        };
        SortingDimensionProvider.prototype.items = function () {
            return this.reordering.apply(this.provider.items());
        };
        SortingDimensionProvider.prototype.localSourceIndex = function (index) {
            return this.reordering.getOldPosition(index);
        };
        SortingDimensionProvider.prototype.localSourceIndices = function (indices) {
            var _this = this;
            if (indices === undefined)
                return undefined;
            return indices.map(function (n) { return _this.reordering.getOldPosition(n); });
        };
        SortingDimensionProvider.prototype.sourceItemsChanged = function () {
            this.update();
        };
        return SortingDimensionProvider;
    }(FilterableDimensionProvider));
    exports.SortingDimensionProvider = SortingDimensionProvider;
});
