var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "jrptlib/Evented", "view/util/ItemFilter"], function (require, exports, Evented, ItemFilter_1) {
    "use strict";
    exports.__esModule = true;
    var FilterableMatrixProvider = (function (_super) {
        __extends(FilterableMatrixProvider, _super);
        function FilterableMatrixProvider(provider, filteredDimensions, scaleOnSelection) {
            var _this = _super.call(this) || this;
            _this.provider = provider;
            _this.scaleOnSelection = scaleOnSelection;
            _this.unitFilter = new ItemFilter_1.ItemFilter(provider.unitCount());
            provider.on("changed", function (event) { return _this.changed(event); });
            _this.dimensions = filteredDimensions;
            _this.dimensions.length = provider.dimensionsCount();
            var _loop_1 = function (dim) {
                if (filteredDimensions[dim]) {
                    filteredDimensions[dim].setOwner(this_1);
                    filteredDimensions[dim].setSelectionChangedHandler(function (major) { return _this.selectionChanged(dim, major); });
                }
            };
            var this_1 = this;
            for (var dim = 0; dim < filteredDimensions.length; dim++) {
                _loop_1(dim);
            }
            return _this;
        }
        FilterableMatrixProvider.prototype.getSource = function () {
            return this.provider;
        };
        FilterableMatrixProvider.prototype.changed = function (event) {
            if (event.unitsChanged) {
                this.unitFilter = new ItemFilter_1.ItemFilter(this.provider.unitCount());
            }
            for (var i = 0; i < this.dimensions.length; i++) {
                var dim = this.dimensions[i];
                if (dim && event.dimensionsChanged[i]) {
                    dim.itemsChanged();
                }
            }
            this.emit("changed", event);
        };
        FilterableMatrixProvider.prototype.selectionChanged = function (dim, major) {
            var unitAffected = this.provider.variableUnit(dim);
            if (unitAffected) {
                unitAffected = this.updateUnitFilter();
            }
            var dimChanged = [];
            dimChanged[dim] = true;
            this.emit("changed", {
                domainChanged: this.scaleOnSelection,
                dimensionsChanged: dimChanged,
                unitsChanged: unitAffected,
                majorChange: major
            });
        };
        FilterableMatrixProvider.prototype.updateUnitFilter = function () {
            var _this = this;
            var indexFilter = this.dimensions.map(function (d, i) { return d && _this.provider.variableUnit(i) ? d.sourceIndices() : undefined; });
            if (indexFilter.every(function (indices) { return indices === undefined; })) {
                if (this.unitFilter.isFull())
                    return false;
                this.unitFilter.addAll();
                return true;
            }
            else {
                var units = this.provider.dataUnits(indexFilter);
                return this.unitFilter.updateSelected(units);
            }
        };
        FilterableMatrixProvider.prototype.dataValue = function (data) {
            return this.provider.dataValue(data);
        };
        FilterableMatrixProvider.prototype.unitCount = function () {
            return this.unitFilter.size();
        };
        FilterableMatrixProvider.prototype.unitLabel = function (unitIndex) {
            return this.provider.unitLabel(this.originalUnit(unitIndex));
        };
        FilterableMatrixProvider.prototype.unitDomain = function (unitIndex) {
            unitIndex = this.originalUnit(unitIndex);
            if (!this.scaleOnSelection) {
                return this.provider.unitDomain(unitIndex);
            }
            else {
                return this.provider.dataDomain(unitIndex, this.selectedIndices());
            }
        };
        FilterableMatrixProvider.prototype.unitScale = function (unitIndex, domain) {
            return this.provider.unitScale(this.originalUnit(unitIndex), domain);
        };
        FilterableMatrixProvider.prototype.dimensionsCount = function () {
            return this.dimensions.length;
        };
        FilterableMatrixProvider.prototype.dimension = function (dim) {
            var d = this.dimensions[dim];
            return d ? d : this.provider.dimension(dim);
        };
        FilterableMatrixProvider.prototype.variableUnit = function (dim) {
            return this.provider.variableUnit(dim);
        };
        FilterableMatrixProvider.prototype.legendDimension = function (dim) {
            return this.dimensions[dim];
        };
        FilterableMatrixProvider.prototype.dataUnit = function (index) {
            return this.unitFilter.filteredIndex(this.provider.dataUnit(this.originalIndex(index)));
        };
        FilterableMatrixProvider.prototype.dataUnits = function (indices) {
            return this.unitFilter.filteredIndices(this.provider.dataUnits(this.originalIndices(indices)));
        };
        FilterableMatrixProvider.prototype.dataText = function (data, index) {
            return this.provider.dataText(data, this.originalIndex(index));
        };
        FilterableMatrixProvider.prototype.dataDomain = function (unitIndex, indices) {
            if (this.scaleOnSelection) {
                indices = this.originalIndices(indices);
            }
            return this.provider.dataDomain(this.originalUnit(unitIndex), indices);
        };
        FilterableMatrixProvider.prototype.getActions = function (index) {
            return this.provider.getActions(this.originalIndex(index));
        };
        FilterableMatrixProvider.prototype.selectedIndices = function () {
            var li = [];
            for (var i = 0; i < this.dimensions.length; i++) {
                var dim = this.dimensions[i];
                li[i] = dim ? dim.sourceIndices() : undefined;
            }
            return li;
        };
        FilterableMatrixProvider.prototype.originalIndex = function (index) {
            var li = [];
            for (var i = 0; i < this.dimensions.length; i++) {
                var dim = this.dimensions[i];
                li[i] = dim ? dim.sourceIndex(index[i]) : index[i];
            }
            return li;
        };
        FilterableMatrixProvider.prototype.originalIndices = function (indices) {
            var li = [];
            for (var i = 0; i < this.dimensions.length; i++) {
                var dim = this.dimensions[i];
                li[i] = dim ? dim.sourceIndices(indices[i]) : undefined;
            }
            return li;
        };
        FilterableMatrixProvider.prototype.originalUnit = function (index) {
            return this.unitFilter.index(index);
        };
        FilterableMatrixProvider.prototype.filteredOrnament = function (ornamentMatrix) {
            var ornaments = new Array(this.dimensions.length);
            for (var dim = 0; dim < this.dimensions.length; dim++) {
                var dimension = this.dimensions[dim];
                if (dimension) {
                    ornaments[dim] = dimension.ornament(ornamentMatrix.ornament(dim));
                }
                else {
                    ornaments[dim] = ornamentMatrix.ornament(dim);
                }
            }
            return {
                ornament: function (dim) {
                    return ornaments[dim];
                }
            };
        };
        return FilterableMatrixProvider;
    }(Evented));
    exports.FilterableMatrixProvider = FilterableMatrixProvider;
    var FilterableTabularProvider = (function (_super) {
        __extends(FilterableTabularProvider, _super);
        function FilterableTabularProvider(provider, filteredDimensions, scaleOnSelection) {
            return _super.call(this, provider, filteredDimensions, scaleOnSelection) || this;
        }
        FilterableTabularProvider.prototype.data = function (index) {
            return this.provider.data(this.originalIndex(index));
        };
        FilterableTabularProvider.prototype.dataConsolidatedDomain = function (unitIndex, indices, consolidatedDim) {
            if (this.scaleOnSelection) {
                indices = this.originalIndices(indices);
            }
            return this.provider.dataConsolidatedDomain(this.originalUnit(unitIndex), indices, consolidatedDim);
        };
        return FilterableTabularProvider;
    }(FilterableMatrixProvider));
    exports.FilterableTabularProvider = FilterableTabularProvider;
    var FilterableLinesProvider = (function (_super) {
        __extends(FilterableLinesProvider, _super);
        function FilterableLinesProvider(provider, filteredDimensions, scaleOnSelection) {
            return _super.call(this, provider, filteredDimensions, scaleOnSelection) || this;
        }
        FilterableLinesProvider.prototype.xLabel = function () {
            return this.provider.xLabel();
        };
        FilterableLinesProvider.prototype.xDomain = function () {
            return this.provider.xDomain();
        };
        FilterableLinesProvider.prototype.xScale = function (domain) {
            return this.provider.xScale(domain);
        };
        FilterableLinesProvider.prototype.data = function (index) {
            return this.provider.data(this.originalIndex(index));
        };
        return FilterableLinesProvider;
    }(FilterableMatrixProvider));
    exports.FilterableLinesProvider = FilterableLinesProvider;
    var FilterableVariableLinesProvider = (function (_super) {
        __extends(FilterableVariableLinesProvider, _super);
        function FilterableVariableLinesProvider() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        FilterableVariableLinesProvider.prototype.setXDomain = function (domain) {
            this.provider.setXDomain(domain);
        };
        FilterableVariableLinesProvider.prototype.getXDomain = function () {
            return this.provider.getXDomain();
        };
        FilterableVariableLinesProvider.prototype.getFixedProvider = function () {
            var ret = this.provider.getFixedProvider();
            return ret;
        };
        return FilterableVariableLinesProvider;
    }(FilterableLinesProvider));
    exports.FilterableVariableLinesProvider = FilterableVariableLinesProvider;
});
