define(["require", "exports"], function (require, exports) {
    "use strict";
    exports.__esModule = true;
    function getOrCreateChild(parent, childName) {
        var child = $(parent).children(childName);
        if (child.length == 0) {
            var n = $.parseXML("<" + childName + "/>").documentElement;
            child = $(n).appendTo($(parent));
        }
        return child;
    }
    var SeriesOptions = (function () {
        function SeriesOptions(series) {
            this.series = [];
            this.legends = [];
            this.paging = [];
            this.filters = [];
            this.sortings = [];
            this.ids = series.ids;
            var size = this.ids.length;
            this.series = new Array(size);
            this.legends = new Array(size);
            this.paging = new Array(size);
            this.filters = new Array(size);
            this.sortings = new Array(size);
        }
        SeriesOptions.prototype.getIds = function () {
            return this.ids;
        };
        SeriesOptions.prototype.getLabels = function () {
            return this.labels;
        };
        SeriesOptions.prototype.validate = function (originalDim) {
            if (originalDim > this.ids.length)
                throw "Unsupported dimensions count " + originalDim + " (max " + this.ids.length + ")";
            for (var d = 0; d < originalDim; d++) {
                var occ = 0;
                for (var s = 0; s < this.series.length; s++) {
                    if (this.series[s] == d)
                        occ++;
                }
                if (occ == 0)
                    throw "Dimension " + d + " is not assigned to a serie.";
                if (occ > 1)
                    throw "Dimension " + d + " is assigned to several series.";
            }
            return null;
        };
        SeriesOptions.prototype.size = function () {
            return this.ids.length;
        };
        SeriesOptions.prototype.cut = function (fromSerie) {
            this.ids = this.ids.slice(0, fromSerie);
            this.series.length = fromSerie;
            this.legends.length = fromSerie;
            this.paging.length = fromSerie;
            this.filters.length = fromSerie;
            this.sortings.length = fromSerie;
        };
        SeriesOptions.prototype.move = function (from, to) {
            this.series[to] = this.series[from];
            this.legends[to] = this.legends[from];
            this.paging[to] = this.paging[from];
            this.filters[to] = this.filters[from];
            this.sortings[to] = this.sortings[from];
            this.series[from] = null;
            this.legends[from] = null;
            this.paging[from] = null;
            this.filters[from] = null;
            this.sortings[from] = null;
        };
        SeriesOptions.prototype.useLegend = function () {
            return this.legends.some(function (l) { return l; });
        };
        SeriesOptions.prototype.legendCount = function () {
            var ret = 0;
            this.legends.forEach(function (l) { if (l)
                ret++; });
            return ret;
        };
        SeriesOptions.prototype.usePaging = function () {
            return this.paging.some(function (l) { return l != null && l > 0; });
        };
        SeriesOptions.prototype.useFilter = function () {
            return this.filters.some(function (l) { return l != null && l > 0; });
        };
        SeriesOptions.prototype.useSorting = function () {
            return this.sortings.some(function (l) { return l; });
        };
        SeriesOptions.prototype.allLegendsOn = function (state) {
            for (var i = 0; i < this.legends.length; i++) {
                this.legends[i] = state;
            }
        };
        SeriesOptions.prototype.allPaging = function (pageCount) {
            for (var i = 0; i < this.paging.length; i++) {
                this.paging[i] = pageCount;
            }
        };
        SeriesOptions.prototype.applyOptions = function (options, dimIds, acceptableDimIds) {
            for (var _i = 0, dimIds_1 = dimIds; _i < dimIds_1.length; _i++) {
                var d = dimIds_1[_i];
                var occ = 0;
                for (var s in options.series) {
                    if (options.series[s].dim == d)
                        occ++;
                }
                if (occ == 0)
                    throw "Dimension " + d + " is not assigned to a serie.";
                if (occ > 1)
                    throw "Dimension " + d + " is assigned to several series.";
            }
            for (var serie in options.series) {
                var serieIndex = this.ids.indexOf(serie);
                if (serieIndex == -1)
                    throw "Serie " + serie + " not defined";
                var val = options.series[serie];
                if (val.dim) {
                    var dimIndex = dimIds.indexOf(val.dim);
                    if (dimIndex != -1) {
                        this.series[serieIndex] = dimIndex;
                        if (val.legend != null) {
                            this.legends[serieIndex] = val.legend;
                        }
                        if (val.paging != null) {
                            this.paging[serieIndex] = val.paging;
                        }
                        if (val.filter != null) {
                            this.filters[serieIndex] = val.filter;
                        }
                        if (val.sorting != null) {
                            this.sortings[serieIndex] = val.sorting;
                        }
                    }
                    else if (acceptableDimIds.indexOf(val.dim) == -1) {
                        throw "Dimension " + val.dim + " not defined";
                    }
                    else {
                        this.series[serieIndex] = null;
                        this.legends[serieIndex] = false;
                        this.paging[serieIndex] = null;
                        this.filters[serieIndex] = null;
                        this.sortings[serieIndex] = false;
                    }
                }
            }
        };
        return SeriesOptions;
    }());
    exports.SeriesOptions = SeriesOptions;
    function nullableBool(value) {
        if (value === null)
            return null;
        return value ? "true" : "false";
    }
    var SeriesOptionsModel = (function () {
        function SeriesOptionsModel() {
            this.series = {};
        }
        SeriesOptionsModel.prototype.getDimension = function (serie) {
            var s = this.series[serie];
            return s ? s.dim : null;
        };
        SeriesOptionsModel.prototype.getSerie = function (dimension) {
            for (var s in this.series) {
                if (this.series[s].dim == dimension)
                    return s;
            }
            return null;
        };
        SeriesOptionsModel.prototype.setDimension = function (serie, dim) {
            var newDim = dim ? this.series[this.getSerie(dim)] : null;
            var oldDim = this.series[serie];
            for (var s in this.series) {
                if (this.series[s].dim == dim) {
                    if (oldDim)
                        this.series[s] = oldDim;
                    else
                        delete this.series[s];
                }
            }
            if (dim) {
                if (newDim)
                    this.series[serie] = newDim;
                else
                    this.series[serie] = { dim: dim, legend: null, paging: null, filter: null, sorting: null };
            }
            else {
                delete this.series[serie];
            }
        };
        SeriesOptionsModel.prototype.save = function (parent) {
            var series = getOrCreateChild(parent, "series");
            series.children().remove();
            for (var s in this.series) {
                var options = this.series[s];
                getOrCreateChild(series, s)
                    .attr("dim", options.dim)
                    .attr("legend", nullableBool(options.legend))
                    .attr("paging", options.paging)
                    .attr("filter", options.filter)
                    .attr("sorting", nullableBool(options.sorting));
            }
        };
        SeriesOptionsModel.create = function (parent) {
            var series = $(parent).children("series");
            if (series.length == 0)
                return null;
            var ret = new SeriesOptionsModel();
            $(series[0]).children().each(function () {
                var legend = $(this).attr("legend");
                var paging = $(this).attr("paging");
                var filter = $(this).attr("filter");
                var sorting = $(this).attr("sorting");
                ret.series[this.nodeName] = {
                    dim: $(this).attr("dim"),
                    legend: legend !== undefined ? legend == "true" : null,
                    sorting: sorting !== undefined ? sorting == "true" : null,
                    paging: paging !== undefined ? +paging : null,
                    filter: filter !== undefined ? +filter : null
                };
            });
            return ret;
        };
        return SeriesOptionsModel;
    }());
    exports.SeriesOptionsModel = SeriesOptionsModel;
});
