require([
    'dojo/on',
    'dojo/dom',
    "dojo/parser",
    'dojo/dom-construct',
    "dojo/_base/array",
    "dojo/request",
    "dojo/json",
    "dojo/store/Memory",
    'dojo/store/JsonRest',
    "dijit/registry",
    "dijit/layout/BorderContainer",
    "dijit/layout/TabContainer",
    "dijit/layout/ContentPane",
    "dijit/form/Button",
    "dijit/Dialog",
    "dijit/Tree",
    "dijit/tree/ObjectStoreModel",
    "js/SessionsStore",
    "dojo/domReady!"
], function (on, dom, parser, domConstruct, arrayUtil, request, JSON, Memory, JsonRest, registry, BorderContainer, TabContainer, ContentPane, Button, Dialog, Tree, ObjectStoreModel, SessionsStore) {
	// create the BorderContainer and attach it to our appLayout div
	var appLayout = new BorderContainer({
	    design: "headline"
	}, "appLayout");
	 
	// create the TabContainer
	var contentTabs = new TabContainer({
	    region: "center",
	    id: "contentTabs",
	    tabPosition: "top",
	    "class": "centerPanel"
	});
	 
	// add the TabContainer as a child of the BorderContainer
	appLayout.addChild( contentTabs );
	 
	// create and add the BorderContainer edge regions
	var topPane = new ContentPane({
        region: "top",
        "class": "edgePanel",
        content: "Execution Statistics"
    });
	appLayout.addChild(topPane);
	
	var switchSessionButton = new Button({
		label: "Switch Session...",
		onClick: function() { switchSession(); }
	});
	topPane.addChild(switchSessionButton);
	
	var leftPane = new ContentPane({
        region: "left",
        id: "leftCol", "class": "edgePanel",
        content: "Reports",
        splitter: true,
        title: "Reports"
    });
	appLayout.addChild(leftPane);
	
	var rightPane;
	
//	// Add initial content to the TabContainer
//	var group1 = contentTabs.addChild(
//	    new ContentPane({
//	        content: "<h4>Bla</h4>",
//	        title: "Group 1"
//	    })
//	);
	
	// start up and do layout
	appLayout.startup();
	
	showReports(leftPane);

	function showReports(pane) {
		request.get("../reports?f=json", {
			// Parse data from JSON to a JavaScript object
			handleAs: "json"
		}).then(
				function(data){
					pane.set('content', buildReportsList(data.reports));
				},
				function(error){
					reportsDiv.innerHTML = error;
				}
		);
	}
	
	function buildReportsList(reports) {
		var ul = domConstruct.create("ul");
		arrayUtil.forEach(reports, function(report,i){
			var li = domConstruct.create("li", null, ul);
			var a = domConstruct.create("a", {
				href: "#",
				innerHTML: report.name,
			}, li);
			on(a, "click", function() { switchReport(report.name); });
		});
		return ul;
	}
	
	function switchReport(id) {
		request.get("../reports/" + id + "?f=json", {
			// Parse data from JSON to a JavaScript object
			handleAs: "json"
		}).then(
				function(data){
					showReport(data);
				},
				function(error){
					reportsDiv.innerHTML = error;
				}
		);
	}
	
	function showReport(report) {
		arrayUtil.forEach(report.pages, function(page,i){
			addPage(page);
		});
	}
	
	function addPage(page) {
		var group = new ContentPane({
	        content: "<h4>" + page.name + "</h4>",
	        title: page.name
	    });
		contentTabs.addChild(group);
		arrayUtil.forEach(page.views, function(view,i){
			addView(view, group);
		});
	}
	
	function addView(view, parent) {
		var section = new BorderContainer({
		    design: "headline"
		});
		var content = new ContentPane({
	        content: "<h4>Yok</h4>",
	        title: view.name
	    });
		var configure = new Button({
			label: "Configure...",
			onClick: function() { configureView(view); }
		});
		content.addChild(configure);
		section.addChild(content);
		parent.addChild(section);
	}
	
	function configureView(view) {
		parser.parse();
		var dialog = registry.byId("configureViewDialog");
		createTreeCounter(dom.byId("countersTree"));
		dialog.isValid = function() {
			return true;
		};
		dialog.show();
	}
	
	function createTreeCounter(div) {
		// Create test store, adding the getChildren() method required by ObjectStoreModel
	    var myStore = new SessionsStore({});
	    var result = myStore.query("", null);
	    alert(JSON.stringify(result));
		var myModel = new ObjectStoreModel({
	        store: myStore,
	        query: {id: ''}
	    });
		var tree = new Tree({
			model: myModel
		});
		tree.placeAt(div);
	}
	
	function switchSession() {
		if (!rightPane) {
			rightPane = new ContentPane({
		        region: "right",
		        id: "rightCol",
		        "class": "edgePanel",
		        content: "Sessions",
		        splitter: true,
		    });
			var myStore = new SessionsStore();
			var myModel = new ObjectStoreModel({
		        store: myStore,
		        query: {id: ''}
		    });
			var tree = new Tree({
				model: myModel
			});
			rightPane.addChild(tree);
			appLayout.addChild(rightPane);
		}
//		parser.parse();
//		var dialog = registry.byId("switchSessionDialog");
//		var div = dom.byId("sessionsTree");
//	    var myStore = new SessionsStore();
////	    myStore.query("", null).then(function(data) {
////	    	alert(JSON.stringify(data));
////	    });
//		var myModel = new ObjectStoreModel({
//	        store: myStore,
//	        query: {id: ''}
//	    });
//		var tree = new Tree({
//			model: myModel
//		});
//		tree.placeAt(div);
//		dialog.show();
	}
	
//	
//    var greetingNode = dom.byId('greeting');
//    domConstruct.place('<i> Dojo!</i>', greetingNode);
//    var reportsDiv = dom.byId('reports');
//    var reportsStore = new JsonRest({target:"../reports/"})
});